// HitObject.cpp: implementation of the HitObject class.
//				This class stores the representation of a hit object
//				This includes the actual point on the object where
//				the hit occured, the normal vector at this point, 
//				and the time t of the hit.
//////////////////////////////////////////////////////////////////////

#include "HitObject.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

HitObject::HitObject()
{
	normal = Vec3(0.0f,0.0f,0.0f);
	hit = Vec4(0.0f,0.0f,0.0f,1.0f);
	t = 0;
}

HitObject::~HitObject()
{
}

HitObject::HitObject(const HitObject& ho)
{
	normal = ho.normal;
	hit = ho.hit;	
	t = ho.t;
}

void HitObject::Set(const HitObject& ho)
{
	normal = ho.normal;
	hit = ho.hit;
	t = ho.t;
}

//////////////////////////////////////////////////////////////////////
// Set: sets the hit object and calculates a normal based on the
// objects center. *Used for spheres*
//////////////////////////////////////////////////////////////////////
void HitObject::Set(const Ray ray, const double T, const Vec4 C)
{
	t = T;

	Vec4 tempV4 ( (ray.v3Dir * t).x, 
			      (ray.v3Dir * t).y, 
				  (ray.v3Dir * t).z,
				  1 ); 

	hit = ray.v4Ori + tempV4;

	Vec3 tempV3 ( (hit - C).x,
		          (hit - C).y,
				  (hit - C).z );

	normal = tempV3;

	normal.Normalize();	
}

//////////////////////////////////////////////////////////////////////
// Set : sets the hit object given an already constructed normal
//////////////////////////////////////////////////////////////////////
void HitObject::Set(const Ray ray, const double T, const Vec3 N)
{
    t = T;
	
	Vec4 tempV4 ( (ray.v3Dir * t).x, 
			      (ray.v3Dir * t).y, 
				  (ray.v3Dir * t).z,
				  1 ); 

    hit = ray.v4Ori + tempV4;
    normal = N;
    normal.Normalize();
}

//////////////////////////////////////////////////////////////////////
// TransformNormal: Multiplies the normal by the inverse transpose
// matrix stored in T.
//////////////////////////////////////////////////////////////////////
Vec3 HitObject::TransformNormal(const Matrix &T) const
{
	Vec3 newNormal (T.MultVector3ByInverseTranspose(normal));
	return newNormal;
}
