// Matrix.h: Interface for the Matrix class.
//			 This class stores a transformation matrix along with it's
//		     inverse and inverse transpose.  It also provides methods
//			 to perform matrix operations.
//			 **NOTE** All trasnformations are applied onto the 
//			 existing matrix.	
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_MATRIX_H__34D36601_7421_455A_B8E8_71FD34A478F9__INCLUDED_)
#define AFX_MATRIX_H__34D36601_7421_455A_B8E8_71FD34A478F9__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Vec3.hpp"
#include "Vec4.hpp"

class Matrix  
{
public:
	double mat[4][4];		// transformation matrix
	double inv[4][4];		// inverse
	double invTrans[4][4];	// transpose inverse

	// Constructors & Destructor
	Matrix();
	virtual ~Matrix();
	
	// resets the matrix to identity
	void setIdentity();

	// scales by by sx,sy,sz
	void scale(double sx, double sy, double sz);

	// translates by tx,tx,tz
	void translate(double tx, double ty, double tz);

	// methods to rotate in each direction by angle
	void rotatex(double angle);
	void rotatey(double angle);
	void rotatez(double angle);

	// Methods to multiply the inverse and inverse
	// transpose matrices by passed in vectors
	Vec3 MultVector3ByInverse (const Vec3 V) const;
	Vec4 MultVector4ByInverse (const Vec4 V);
	Vec3 MultVector3ByInverseTranspose (const Vec3 V) const;

private: 
	// calculates the inverse and inverse tranpose of mat
	void invert();

	// Multiples mat by the passed in 4x4 matrix
	void MultMat(double m2[4][4]);

	// determinant of a 4x4 matrix
	double det4x4( double m[4][4] );

	// determinant of a 3x3 matrix
	double det3x3( double a1, double a2, double a3, double b1, double b2, double b3, double c1, double c2, double c3 );

	// determinant of a 2x2 matrix
	double det2x2( double a, double b, double c, double d);

	void adjoint();
};

#endif // !defined(AFX_MATRIX_H__34D36601_7421_455A_B8E8_71FD34A478F9__INCLUDED_)

