// Object.cpp: Implementation of the Object class.
//			 This is a basis class for all objects supported by the 
//			 ray tracer.  Each object contains a surfaceModel
//			 and transformation matrix.
//////////////////////////////////////////////////////////////////////

#include "Object.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Object::Object()
{
}

Object::~Object()
{
}


//////////////////////////////////////////////////////////////////////
// Returns the surface's Refraction index
//////////////////////////////////////////////////////////////////////
double Object::GetRefractionIndex()
{
    return surfaceModel.RefractionIndex;
}

//////////////////////////////////////////////////////////////////////
// Returns the surface's Specular Exponent index
//////////////////////////////////////////////////////////////////////
double Object::GetSpecExponent()
{
	return surfaceModel.SpecExponent;
}

//////////////////////////////////////////////////////////////////////
// Returns the surface's Transparency value
//////////////////////////////////////////////////////////////////////
double Object::GetTransparent()
{
	return surfaceModel.Transparent;
}

//////////////////////////////////////////////////////////////////////
// Returns the surface's Reflective Value
//////////////////////////////////////////////////////////////////////
double Object::GetReflective()
{
	return surfaceModel.Reflective;
}

//////////////////////////////////////////////////////////////////////
// Returns the surface's Diffuse Contribution
//////////////////////////////////////////////////////////////////////
Colour Object::GetDiffuse()
{
	return surfaceModel.Diffuse;
}

//////////////////////////////////////////////////////////////////////
// Returns the surface's Ambient Contribution
//////////////////////////////////////////////////////////////////////
Colour Object::GetAmbient()
{
    return surfaceModel.Ambient;
}

//////////////////////////////////////////////////////////////////////
// Returns the surface's Specular Contribution
//////////////////////////////////////////////////////////////////////
Colour Object::GetSpecular()
{
    return surfaceModel.Specular;
}

//////////////////////////////////////////////////////////////////////
// Sets the SurfaceModel of this object to surface
//////////////////////////////////////////////////////////////////////
void Object::SetSurface(SurfaceModel surface)
{
	surfaceModel = surface;	
}

