// Object.h: Interface for the Object class.
//			 This is a basis class for all objects supported by the 
//			 ray tracer.  Each object contains a surfaceModel
//			 and transformation matrix.
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_OBJECT_H__88D23CE4_EB40_4536_9B1A_0AE0A5ACE305__INCLUDED_)
#define AFX_OBJECT_H__88D23CE4_EB40_4536_9B1A_0AE0A5ACE305__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "HitObject.h"
#include "SurfaceModel.h"
#include "Matrix.h"

class Object  
{
public:
	SurfaceModel surfaceModel;	// object's surface properties
	Matrix transMatrix;		    // object's transformation matrix

	// Constructor & Desctructor
	Object();
	virtual ~Object();

	// methods to get the surface properties 
	// of this object
    double GetRefractionIndex();
    double GetSpecExponent();
    double GetTransparent();
    double GetReflective();
	Colour GetDiffuse();
    Colour GetAmbient();
    Colour GetSpecular();
	
	// assign the surface 
	void SetSurface(SurfaceModel surface);

	// intersect method which must be overridden by
	// child classes
	virtual bool Intersect(Ray ray, HitObject& ho) = 0;

};

#endif // !defined(AFX_OBJECT_H__88D23CE4_EB40_4536_9B1A_0AE0A5ACE305__INCLUDED_)

