// Polygon.cpp: implementation of the Polygon class.
//			 Parent Class: Object
//			 This class stores the representation of a Polygon object	
//			 and provides a method to intersect a ray with the polygon
//////////////////////////////////////////////////////////////////////

#include "Polygon.h"
#include "Misc.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Polygon::Polygon()
{
	N = 3;
	P = new Vec4[N];
	
	P[0].x = 1;
	P[0].y = 0;
	P[0].z = 0;

	P[1].x = 0;
	P[1].y = 1;
	P[0].z = 0;

	P[2].x = -1;
	P[2].y = 0;
	P[2].z = 0;
}

Polygon::~Polygon()
{
	delete []P;
}

Polygon::Polygon(const Polygon& poly)
{
	N = poly.N;
}

Polygon::Polygon(const int n, const Vec4* p)
{
	int i;

	N = n;

	P = new Vec4[N];

	for (i = 0; i < n; i++)
	{
		P[i] = p[i];
	}
}


//////////////////////////////////////////////////////////////////////
// Set Methods
//////////////////////////////////////////////////////////////////////
void Polygon::Set(const Polygon& poly)
{
}

void Polygon::Set(const int n, const Vec4* p)
{
}

//////////////////////////////////////////////////////////////////////
// PointIsInside : This function implements the Jordan-curve test.
// It returns 1 if the point is inside the polygon and 0 if it 
// is outside.
//////////////////////////////////////////////////////////////////////
bool Polygon::PointIsInside(Vec4 pt) {
  int i, j;
  bool c;
    
  double x = pt.x;
  double y = pt.y;

  c = false;
  
  for (i = 0, j = N-1; i < N; j = i++) {
    /*
     * Flip a bit for each edge that:
     *   straddles the x-axis (going up or going down), and
     *   intersects the x-axis at a positive value of x.
     */
    if ((((P[i].y <= y) && (y < P[j].y)) ||
	 ((P[j].y <= y) && (y < P[i].y))) &&
	(x < (P[j].x - P[i].x) * (y - P[i].y) / (P[j].y - P[i].y) + P[i].x))

      c = !c;
  }

  return c;
}


//////////////////////////////////////////////////////////////////////
// Intersect : Tests for intersection with given ray.  If there
// is an intersection, the hit information is stored in ho, and true
// is returned, otherwise false is returned.
//////////////////////////////////////////////////////////////////////
bool Polygon::Intersect(Ray ray, HitObject& ho)
{
	// transform the ray according to this objects transformation
	// matrix
	Ray newRay = ray.InverseTransRay(this->transMatrix);

	// get two vectors on the polygon
	Vec3 e1 ( (P[1]-P[0]).x,
			  (P[1]-P[0]).y,
			  (P[1]-P[0]).z );

	Vec3 e2 ( (P[2]-P[0]).x, 
			  (P[2]-P[0]).y,
			  (P[2]-P[0]).z );

	Vec3 tvec ( (newRay.v4Ori - P[0]).x,
				(newRay.v4Ori - P[0]).y,
				(newRay.v4Ori - P[0]).z );
				
		
	Vec3 pvec, qvec, dir;
	double det, inv_det;
	double t;
	
	dir = newRay.v3Dir;
	pvec = dir/e2;		
	det = e1*pvec;		

	if( det < 0.0 )
		return false;
	
	qvec = tvec/e1;
		
	t = e2 * qvec;
	
	inv_det = 1.0/det;

	t *= inv_det;
	
	// get normal - cross product of e1 and e2
    Vec3 normal (e1/e2);
	
	// temporary hit object to test for where the
	// hitpoint is in respect to our polygon boundaries
	HitObject hOriRay;
	hOriRay.Set(newRay, t, normal);
	
	// if the hitpoint is inside the polygon, then
	// we set our object hitpoint and return true
	if (PointIsInside(hOriRay.hit) == true)
	{
		ho.Set(ray, t, normal);
		return true;
	}
	else
		return false;
}

