// Vec3.h: 3D Vector
//			 This class contains the definition and implementation
//			 of 3 dimensional vectors. Contains overloaded operators
//			 for most vector operations, plus additional methods 
//			 for normalization and length
//////////////////////////////////////////////////////////////////////

#ifndef _VEC3_H_
#define _VEC3_H_

#include <math.h>

#define sqr(x)	((x)*(x))

class Vec3
{
public:
	// 3D Vector Representation
	double x, y, z;

	///////////////////////////////////
	// Constructors/Set Methods
	///////////////////////////////////
	Vec3() 
	{ x = 0.0f; y = 0.0f; z = 0.0f; };
	
	Vec3(const double X, const double Y, const double Z) 
	{ x = X; y = Y; z = Z; };

	void Set(const double X, const double Y, const double Z)
	{ x = X; y = Y; z = Z; };

	void Set(const Vec3 &V)
	{ x = V.x; y = V.y; z = V.z; };

	////////////////////////////////////
	// 3D Vector Operations
	////////////////////////////////////

	// COMPARISON (==)
    int operator == (const Vec3& A)              
      { return (x==A.x && y==A.y && z==A.z); }

	// ASSIGNMENT (=)
    Vec3& operator = (const Vec3& A)            
      { x=A.x; y=A.y; z=A.z; 
        return(*this);  };

	// ADDITION (+)
    Vec3 operator + (const Vec3& A) const       
      { Vec3 Sum(x+A.x, y+A.y, z+A.z); 
        return(Sum); };

	// SUBTRACTION (-)
    Vec3 operator - (const Vec3& A) const       
      { Vec3 Diff(x-A.x, y-A.y, z-A.z);
        return(Diff); };

	// DOT-PRODUCT (*)
    double operator * (const Vec3& A) const       
      { double DotProd = x*A.x+y*A.y+z*A.z; 
        return(DotProd); };

	// CROSS-PRODUCT (/)
    Vec3 operator / (const Vec3& A) const       
      { Vec3 CrossProd(y*A.z-z*A.y, z*A.x-x*A.z, x*A.y-y*A.x);
        return(CrossProd); };

	// MULTIPLY BY SCALAR V*s (*)
    Vec3 operator * (const double s) const        
      { Vec3 Scaled(x*s, y*s, z*s); 
        return(Scaled); };

	// DIVIDE BY SCALAR (/)
    Vec3 operator / (const double s) const        
      { Vec3 Scaled(x/s, y/s, z/s);
        return(Scaled); };

	// SCALAR MULT s*V
    friend inline Vec3 operator *(double s, const Vec3& v)  
      { return Vec3(v.x*s, v.y*s, v.z*s); }

	// ACCUMULATED VECTOR ADDITION (+=)
    void operator += (const Vec3 A)              
      { x+=A.x; y+=A.y; z+=A.z; };

	// ACCUMULATED VECTOR SUBTRACTION (+=)
    void operator -= (const Vec3 A)              
      { x-=A.x; y-=A.y; z-=A.z; };

	// ACCUMULATED SCALAR MULT (*=)
    void operator *= (const double s)              
      { x*=s; y*=s; z*=s; };

	// ACCUMULATED SCALAR DIV (/=)
    void operator /= (const double s)              
      { x/=s; y/=s; z/=s; };

	// NEGATION (-)
    Vec3 operator - (void) const                 
      { Vec3 Negated(-x, -y, -z);
        return(Negated); };

	// Length
	double Length ()
	{ return ((double)sqrt(sqr(x)+sqr(y)+sqr(z))); };

	// normalizes the vectors
	void Normalize ()
	{ double L = Length();               
      if (L>0) { x/=L; y/=L; z/=L; } };

};


#endif
